'-----------------------------------------------------------------------------
'--- File: export.access.template.vbs
'--- Description: This VBScript example exports the table "mytable" from the
'---   MS Access mdb at c:\myexisting.mdb to sasuser.mydata using the IOM
'---   provider.  You will have to change the values in the "Configuration
'---   vars" section to identify your dataset, connection, mdb file, etc. 
'---   Nothing else should need to be changed.
'-----------------------------------------------------------------------------

Option Explicit

'-----------------------------------------------------------------------------
'--- Constants

const ERR_NOSASRECORDSET = -1
const ERR_NOACCESSRECORDSET = -2
const adOpenForwardOnly = 0
const adOpenDynamic = 2
const adLockReadOnly = 1
const adLockBatchOptimistic = 4
const adCmdTableDirect = 512
const adCmdText = 1
const adSchemaTables = 20
const adChar = 129
const adVarChar = 200
const adWChar = 130
const adVarWChar = 202
const adDouble = 5

'-----------------------------------------------------------------------------
'--- Configuration vars

Dim Filename, SASConnectionString, TableName, DatasetName, DatasetType
'--- Full path to the .mdb file
Filename = "c:\myexisting.mdb" 

'--- Connection string for SAS OLE DB provider
SASConnectionString = "Provider=SAS.IOMProvider.1;Data Source=_LOCAL_" 

'--- Access tablename
TableName = "sashelp_adomsg"

'--- libref.member
DatasetName = "sasuser.mydata" 

'--- DatasetType should be adCmdTableDirect.
DatasetType = adCmdTableDirect

'-----------------------------------------------------------------------------
'--- Global vars

Dim AccessConnectionString
AccessConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & Filename

'-----------------------------------------------------------------------------
'--- GetAccessRecordset: Creates and opens a Recordset object on an existing SAS dataset

Private Function GetAccessRecordset()
  Dim Connection
  Set Connection = WScript.CreateObject("ADODB.Connection")
  Set GetAccessRecordset = WScript.CreateObject("ADODB.Recordset")

  Connection.Open AccessConnectionString
  GetAccessRecordset.Open TableName, Connection, adOpenForwardOnly, _
                          adLockReadOnly, DatasetType
End Function

'-----------------------------------------------------------------------------
'--- GetSASRecordset: Creates and opens a Recordset object on a newly
'---   created Access table.  If a table of the same name already exists then
'---   the user is prompted to overwrite it or cancle the import operation.

Private Function GetSASRecordset( AccessRecordset )
  Dim Connection, TablesRS, Tbl, res, bFirst, bContinue, fld, sql, Name
  Set Connection = WScript.CreateObject("ADODB.Connection")
  Set GetSASRecordset = WScript.CreateObject("ADODB.Recordset")
  Set TablesRS = WScript.CreateObject("ADODB.Recordset")

  Connection.Open SASConnectionString
  Set TablesRS = Connection.Execute( _
      "select libname, memname from dictionary.tables")

  bContinue = True
  While bContinue And Not TablesRS.EOF
    If lcase(TablesRS.Fields("libname").Value & _
             "." & _
             TablesRS.Fields("memname")) = lcase(DatasetName) Then
      res = MsgBox( DatasetName & " table found." & vbNewLine & _
                    "Click ""Ok"" to continue and overwrite this table.", _
                    vbOKCancel Or vbExclamation, "SAS Import Data Wizard")
      If res = vbCancel Then
        Set GetSASRecordset = Nothing
        Exit Function
      End If
      Connection.Execute "DROP TABLE " & DatasetName
      bContinue = False
     End If
    TablesRS.MoveNext
  WEnd

  bFirst = True
  sql = "CREATE TABLE " & DatasetName & " ("
  For Each fld In AccessRecordset.Fields
    If Not bFirst Then sql = sql & ", "
    sql = sql & fld.name
    If fld.Type = adChar Or _
       fld.Type = adVarChar Or _
       fld.Type = adWChar Or _
       fld.Type = adVarWChar Then
      sql = sql & " CHAR(" & fld.DefinedSize & ")"
    ElseIf fld.Type = adDouble Then
      sql = sql & " NUM"
    End If
    bFirst = False
  Next
  sql = sql & ")"

  Connection.Execute sql
    
  GetSASRecordset.Open DatasetName, Connection, adOpenDynamic, _
                       adLockBatchOptimistic, adCmdTableDirect

End Function

'-----------------------------------------------------------------------------
'--- CopyData: Copies all data from src (an open Recordset object) to dst (an
'---   open Recordset object)

Private Sub CopyData( src, dst )
  Dim col
  While Not src.EOF
    dst.AddNew
    For col = 0 To src.Fields.Count - 1
      dst.Fields(col).Value = src.Fields(col).Value
    Next
    dst.UpdateBatch
    src.MoveNext
  Wend
End Sub

'-----------------------------------------------------------------------------
'--- Main: The main controling routine.  Opens Recordset objects on an
'---   existing SAS dataset and a new Access table and then copies the SAS
'---   dataset into the Access table.

Private Sub Main()
  Dim SASRecordset
  Dim AccessRecordset

  Set AccessRecordset = GetAccessRecordset()
  If AccessRecordset Is Nothing Then
    WScript.Quit( ERR_NOACCESSRECORDSET )
  End If

  Set SASRecordset = GetSASRecordset( AccessRecordset )
  If SASRecordset Is Nothing Then
    WScript.Quit( ERR_NOSASRECORDSET )
  End If

  CopyData AccessRecordset, SASRecordset

  SASRecordset.Close
  AccessRecordset.Close
End Sub

'-----------------------------------------------------------------------------
'--- Start the Main Sub

Main

