'-----------------------------------------------------------------------------
'--- File: import.access.template.vbs
'--- Description: This VBScript example imports the "sashelp.adomsg" dataset
'---   into the MS Access mdb at "c:\myexisting.mdb".  You will have to change
'---   the values in the "Configuration vars" section to identify your
'---   dataset, connection, and mdb file.  Nothing else should need to be
'---   changed.
'-----------------------------------------------------------------------------

Option Explicit

'-----------------------------------------------------------------------------
'--- Constants

const ERR_NOSASRECORDSET = -1
const ERR_NOACCESSRECORDSET = -2
const adOpenForwardOnly = 0
const adOpenDynamic = 2
const adLockReadOnly = 1
const adLockBatchOptimistic = 4
const adCmdTableDirect = 512
const adCmdText = 1
const adSchemaTables = 20
const adChar = 129
const adVarChar = 200
const adWChar = 130
const adVarWChar = 202
const adDouble = 5

'-----------------------------------------------------------------------------
'--- Configuration vars
Dim Filename, SASConnectionString, DatasetName, DatasetType

'--- Full path to the .mdb file
Filename = "c:\myexisting.mdb" 

'--- Connection string for SAS OLE DB provider
SASConnectionString = "Provider=SAS.IOMProvider.1;Data Source=_LOCAL_" 

'--- libref.member or SELECT SQL query
DatasetName = "sashelp.adomsg" 

'--- If DatasetName is libref.member then DatasetType should be adCmdTableDirect.
'--- If DatasetName is SELECT SQL statement then DatasetType should be adCmdText.
DatasetType = adCmdTableDirect

'-----------------------------------------------------------------------------
'--- Global vars

Dim AccessConnectionString
AccessConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & Filename

'-----------------------------------------------------------------------------
'--- GetSASRecordset: Creates and opens a Recordset object on an existing SAS
'--- dataset

Private Function GetSASRecordset()
  Dim Connection
  Set Connection = WScript.CreateObject("ADODB.Connection")
  Set GetSASRecordset = WScript.CreateObject("ADODB.Recordset")

  Connection.Open SASConnectionString
  GetSASRecordset.ActiveConnection = Connection
  GetSASRecordset.CacheSize = 55
  GetSASRecordset.Properties("Maximum Open Rows") = 110
  GetSASRecordset.Open DatasetName, , adOpenForwardOnly, _
                       adLockReadOnly, DatasetType
End Function

'-----------------------------------------------------------------------------
'--- ConvertToAccessName: replaces '.' characters in str with '_' characters

Private Function ConvertToAccessName( str )
  Dim pos
  ConvertToAccessName = str
  pos = InStr( ConvertToAccessName , "." )
  While Not IsNull( pos ) And pos <> 0
    ConvertToAccessName = Mid( ConvertToAccessName , 1, pos-1 ) & "_" & Mid( ConvertToAccessName, pos+1 )
    pos = InStr( ConvertToAccessName, "." )
  WEnd
End Function

'-----------------------------------------------------------------------------
'--- GetAccessRecordset: Creates and opens a Recordset object on a newly
'---   created Access table.  If a table of the same name already exists then
'---   the user is prompted to overwrite it or cancle the import operation.

Private Function GetAccessRecordset( SASRecordSet )
  Dim Connection, TablesRS, Tbl, res, bFirst, bContinue, fld, sql, Name
  Name = ConvertToAccessName( DatasetName )
  Set Connection = WScript.CreateObject("ADODB.Connection")
  Set GetAccessRecordset = WScript.CreateObject("ADODB.Recordset")
  Set TablesRS = WScript.CreateObject("ADODB.Recordset")

  Connection.Open AccessConnectionString
  Set TablesRS = Connection.OpenSchema( adSchemaTables )

  bContinue = True
  While bContinue And Not TablesRS.EOF
    If TablesRS.Fields("TABLE_NAME").Value = Name Then
      res = MsgBox( Name & " table found." & vbNewLine & _
                    "Click ""Ok"" to continue and overwrite this table.", _
                    vbOKCancel Or vbExclamation, "SAS Import Data Wizard" )
      If res = vbCancel Then
        Set GetAccessRecordset = Nothing
        Exit Function
      End If
      Connection.Execute "DROP TABLE [" & Name & "]"
      bContinue = False
     End If
    TablesRS.MoveNext
  WEnd

  bFirst = True
  sql = "CREATE TABLE [" & Name & "] ("
  For Each fld In SASRecordSet.Fields
    If Not bFirst Then sql = sql & ", "
    sql = sql & "[" & fld.name & "]"
    If fld.Type = adChar Or _
       fld.Type = adVarChar Or _
       fld.Type = adWChar Or _
       fld.Type = adVarWChar Then
      sql = sql & " TEXT(" & fld.DefinedSize & ")"
    ElseIf fld.Type = adDouble Then
      sql = sql & " FLOAT"
    End If
    bFirst = False
  Next
  sql = sql & ")"

  Connection.Execute sql
    
  GetAccessRecordset.Open Name, Connection, adOpenDynamic, _
                          adLockBatchOptimistic, adCmdTableDirect

End Function

'-----------------------------------------------------------------------------
'--- CopyData: Copies all data from src (an open Recordset object) to dst (an
'---   open Recordset object)

Private Sub CopyData( src, dst )
  Dim col
  While Not src.EOF
    dst.AddNew
    For col = 0 To src.Fields.Count - 1
      dst.Fields(col).Value = src.Fields(col).Value
    Next
    dst.UpdateBatch
    src.MoveNext
  Wend
End Sub

'-----------------------------------------------------------------------------
'--- Main: The main controling routine.  Opens Recordset objects on an
'---   existing SAS dataset and a new Access table and then copies the SAS
'---   dataset into the Access table.

Private Sub Main()
  Dim SASRecordset
  Dim AccessRecordset

  Set SASRecordset = GetSASRecordset()
  If SASRecordset Is Nothing Then
    WScript.Quit( ERR_NOSASRECORDSET )
  End If

  Set AccessRecordset = GetAccessRecordset( SASRecordset )
  If AccessRecordset Is Nothing Then
    WScript.Quit( ERR_NOACCESSRECORDSET )
  End If

  CopyData SASRecordset, AccessRecordset

  SASRecordset.Close
  AccessRecordset.Close
End Sub

'-----------------------------------------------------------------------------
'--- Start the Main Sub

Main

