'-----------------------------------------------------------------------------
'--- File: import.excel.template.vbs
'--- Description: This VBScript example imports the "sashelp.adomsg" dataset
'---   into the MS Excel spreadsheet at "c:\myexisting.xls".  You will have to
'---   change the values in the "Configuration vars" section to identify your
'---   dataset, connection, and xls file.  Nothing else should need to be
'---   changed.
'-----------------------------------------------------------------------------

Option Explicit

'-----------------------------------------------------------------------------
'--- Constants

const ERR_NOBOOK = -1
const ERR_NOSHEET = -2
const ERR_NORECORDSET = -3
const adOpenForwardOnly = 0
const adLockReadOnly = 1
const adCmdTableDirect = 512
const adCmdText = 1
const xlThin = 2
const xlEdgeBottom = 9

'-----------------------------------------------------------------------------
'--- Configuration vars
Dim Filename, SASConnectionString, DatasetName, DatasetType

'--- Full path to the .xls file
Filename = "C:\myexisting.xls"

'--- Connection string for SAS OLE DB provider
SASConnectionString = "Provider=SAS.IOMProvider.1;Data Source=_LOCAL_" 

'--- libref.member or SELECT SQL query
DatasetName = "sashelp.adomsg"

'--- If DatasetName is libref.member then DatasetType should be adCmdTableDirect.
'--- If DatasetName is SELECT SQL statement then DatasetType should be adCmdText.
DatasetType = adCmdTableDirect

'-----------------------------------------------------------------------------
'--- GetBook: Returns the opened workbook specified by Filename.
'---   Prompts the user if Filename is the empty string.

Private Function GetBook( appXL )
  If Filename = "" Then
    Set GetBook = appXL.Workbooks.Open( appXL.GetOpenFilename() )
  Else
    Set GetBook = appXL.Workbooks.Open( Filename )
  End If
End Function

'-----------------------------------------------------------------------------
'--- GetSheet: Creates a sheet named DatasetName + " data" in the workbook
'---   Book.  If a sheet of the same name already exists then the user is
'---   prompted to overwrite it or cancle the import operation.

Private Function GetSheet( Book )
  Dim res
  on error resume next
  Set GetSheet = Nothing
  Set GetSheet = Book.Sheets( DatasetName & " data" )
  on error goto 0
  If not GetSheet Is Nothing Then
    res = MsgBox( DatasetName & " worksheet found." & vbNewLine & _
                  "Click ""Ok"" to continue and overwrite data on this sheet.", _
                  vbOKCancel Or vbExclamation, "SAS Import Data Wizard" )
    If res = vbCancel Then
        Set GetSheet = Nothing
    end if
  else
    Set GetSheet = Book.Sheets.Add()
    GetSheet.Name = DatasetName & " data"
  End If
End Function

'-----------------------------------------------------------------------------
'--- GetRecordset: Opens and returns an ADO Recordset on the dataset specified
'--- by DatasetName.

Private Function GetRecordset()
  Dim Connection
  Set Connection = WScript.CreateObject("ADODB.Connection")
  Set GetRecordset = WScript.CreateObject("ADODB.Recordset")

  Connection.Open SASConnectionString
  GetRecordset.Open DatasetName, Connection, _
      adOpenForwardOnly, adLockReadOnly, DatasetType
End Function

'-----------------------------------------------------------------------------
'--- CopyData: Copies all data from Recordset (an open Recordset object) to
'--- Sheet (an open Excel Sheet object)

Private Sub CopyData( Sheet, Recordset )
  Dim maxRows, maxCols
  Dim i, j
  Dim Rng
  maxRows = 65536
  maxCols = 256
  Set Rng = Sheet.Range("a1")
  j = 0
  While j < Recordset.Fields.Count and j < maxCols
    Rng.Offset(0, j).value = Recordset.Fields(j).name
    Rng.Offset(0, j).Borders.Item(xlEdgeBottom).Weight = xlThin
    j = j + 1
  WEnd
  i = 1
  While Not Recordset.EOF and i < maxRows
    Set Rng = Rng.Offset(1)
    j = 0
    While j < Recordset.Fields.Count and j < maxCols
      Rng.Offset(, j).Value = Recordset.Fields(j).Value
      j = j + 1
    WEnd
    Recordset.MoveNext
    i = i + 1
  WEnd
  Sheet.Columns.AutoFit
End Sub

'-----------------------------------------------------------------------------
'--- Main: The main controling routine.  Opens a Recordset object on an
'---   existing SAS dataset and a new Sheet in an Excel spreadsheet and then
'---   copies the SAS dataset into the Sheet.

Private Sub Main()
  Dim appXL
  Dim Book, Sheet
  Dim Recordset

  Set appXL = WScript.CreateObject("Excel.Application")

  Set Book = GetBook( appXL )
  if Book Is Nothing Then WScript.Quit( ERR_NOBOOK )

  Set Sheet = GetSheet( Book )
  If Sheet Is Nothing Then
    Book.Close false
    WScript.Quit( ERR_NOSHEET )
  End If

  Set Recordset = GetRecordset()
  If Recordset Is Nothing Then
    Book.Close false
    WScript.Quit( ERR_NORECORDSET )
  End If

  CopyData Sheet, Recordset

  Recordset.Close
  Book.Close true
End Sub

'-----------------------------------------------------------------------------
'--- Start the Main Sub

Main

